package skyjed.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.*;

import skyjed.SkyJEd;
import skyjed.save.SaveFile;
import skyjed.util.ExceptionHandler;

public class MainWindow {

    private JFrame frame;
    private JMenuItem mntmSave;
    private JMenuItem mntmSaveAs;
    private SaveEditPanel saveEditPanel;
    private QuickSaveEditPanel qSaveEditPanel;
    private AdventureLogEditPanel advLogEditPanel;
    private JTabbedPane tabbedPane;

    private File curFile;
    private SaveFile curSave;

    private ActionListener menuActionListener = new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
            JFileChooser jfc;
            int returnval;
            switch(ev.getActionCommand()) {
                case "File->Open":
                    jfc = new JFileChooser(new File("."));
                    returnval = jfc.showOpenDialog(frame);
                    if(returnval == JFileChooser.APPROVE_OPTION) {
                        open(jfc.getSelectedFile());
                    }
                    break;
                case "File->Save":
                    save(curFile);
                    break;
                case "File->SaveAs":
                    jfc = new JFileChooser(new File("."));
                    returnval = jfc.showSaveDialog(frame);
                    if(returnval == JFileChooser.APPROVE_OPTION) {
                        save(jfc.getSelectedFile());
                    }
                    break;
                case "File->Exit":
                    frame.dispose();
                    break;
                case "Tools->Truncate":
                    jfc = new JFileChooser(new File("."));
                    returnval = jfc.showOpenDialog(frame);
                    if(returnval == JFileChooser.APPROVE_OPTION) {
                        String msg = String.format(
                                "Are you sure you want to truncate this file: %s\n" +
                                "Warning: this will permanently erase any excess data in this file",
                                jfc.getSelectedFile().getAbsolutePath());
                        if(JOptionPane.showConfirmDialog(frame, msg, "Are you sure?",
                                JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE) == JOptionPane.OK_OPTION) {
                            try {
                                SkyJEd.truncateFile(jfc.getSelectedFile());
                            } catch(IOException e) {
                                ExceptionHandler.handleException(e, frame);
                            }
                        }
                    }
                    break;
                case "Help->About":
                    new AboutDialog();
                    break;
            }
        }
    };

    protected void open(File selectedFile) {
        try {
            closeCurrentFile();
            if(!SkyJEd.testLoadSave(selectedFile)) {
                JOptionPane.showMessageDialog(frame, "" +
                        "There was an error during self test\n" +
                        "this usually means failed checksum or\n" +
                        "strange characters in pokemon names.\n" +
                        "Be aware that even just loading and saving file\n" +
                        "may irreversibly break it. Always make backups.",
                        "Self test error", JOptionPane.WARNING_MESSAGE);
            }
            curFile = selectedFile;
            curSave = SkyJEd.openFile(selectedFile);
            mntmSave.setEnabled(true);
            mntmSaveAs.setEnabled(true);
            saveEditPanel = new SaveEditPanel(curSave.slot1);
            if(curSave.qslot != null) {
                qSaveEditPanel = new QuickSaveEditPanel(curSave.qslot);
            }
            advLogEditPanel = new AdventureLogEditPanel(curSave.slot1.adventureLog);
            tabbedPane = new JTabbedPane();
            tabbedPane.add("save", saveEditPanel);
            tabbedPane.add("quicksave", qSaveEditPanel);
            if(curSave.qslot == null) {
                tabbedPane.setEnabledAt(tabbedPane.getTabCount()-1, false);
            }
            tabbedPane.add("adventure log", advLogEditPanel);
            frame.getContentPane().add(tabbedPane, BorderLayout.CENTER);
            frame.pack();
            frame.setMinimumSize(frame.getSize());
        } catch (IOException e) {
            closeCurrentFile();
            ExceptionHandler.handleException(e, frame);
        }
    }

    protected void closeCurrentFile() {
        curFile = null;
        curSave = null;
        mntmSave.setEnabled(false);
        mntmSaveAs.setEnabled(false);
        if(tabbedPane != null) {
            frame.getContentPane().remove(tabbedPane);
        }
        saveEditPanel = null;
        qSaveEditPanel = null;
        advLogEditPanel = null;
        tabbedPane = null;
        frame.validate();
        frame.repaint();
    }

    protected void save(File selectedFile) {
        try {
            curFile = selectedFile;
            curSave.slot1 = saveEditPanel.getSave();
            if(qSaveEditPanel != null) {
                curSave.qslot = qSaveEditPanel.getSave();
            }
            curSave.slot1.adventureLog = advLogEditPanel.getSave();
            SkyJEd.saveFile(selectedFile, curSave);
        } catch (IOException e) {
            ExceptionHandler.handleException(e, frame);
        }
    }

    /**
     * Create the application.
     */
    public MainWindow() {
        initialize();
        frame.setVisible(true);
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        frame = new JFrame();
        frame.setTitle("SkyJEd");
        frame.setSize(300, 200);
        frame.setLocationByPlatform(true);
        //frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JMenuBar menuBar = new JMenuBar();
        frame.setJMenuBar(menuBar);

        JMenu mnFile = new JMenu("File");
        menuBar.add(mnFile);

        JMenuItem mntmOpen = new JMenuItem("Open");
        mntmOpen.setActionCommand("File->Open");
        mntmOpen.addActionListener(this.menuActionListener);
        mnFile.add(mntmOpen);

        mntmSave = new JMenuItem("Save");
        mntmSave.setEnabled(false);
        mntmSave.setActionCommand("File->Save");
        mntmSave.addActionListener(this.menuActionListener);
        mnFile.add(mntmSave);

        mntmSaveAs = new JMenuItem("Save As");
        mntmSaveAs.setEnabled(false);
        mntmSaveAs.setActionCommand("File->SaveAs");
        mntmSaveAs.addActionListener(this.menuActionListener);
        mnFile.add(mntmSaveAs);

        JMenuItem mntmExit = new JMenuItem("Exit");
        mntmExit.setActionCommand("File->Exit");
        mntmExit.addActionListener(this.menuActionListener);
        mnFile.add(mntmExit);

        JMenu mnTools = new JMenu("Tools");
        menuBar.add(mnTools);

        JMenuItem mntmTruncate = new JMenuItem("Truncate File");
        mntmTruncate.setActionCommand("Tools->Truncate");
        mntmTruncate.addActionListener(this.menuActionListener);
        mnTools.add(mntmTruncate);

        JMenu mnHelp = new JMenu("Help");
        menuBar.add(mnHelp);

        JMenuItem mntmAbout = new JMenuItem("About");
        mntmAbout.setActionCommand("Help->About");
        mntmAbout.addActionListener(this.menuActionListener);
        mnHelp.add(mntmAbout);
    }

}
