package skyjed.save;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;

import skyjed.buffer.BooleanBuffer;
import skyjed.buffer.BooleanBufferArray;
import skyjed.util.BitConverterLE;
import skyjed.util.SkyCharConv;
import skyjed.util.SkyStringTrash;

public class SaveSlot {

    private static final int MAGIC_OFFSET = 0x20;
    private static final int MAGIC_BYTELENGTH = 13;
    private static final byte[] MAGIC_DATA = "POKE_DUN_SORA".getBytes(StandardCharsets.UTF_8);

    private static final int PKMN_STORAGE_OFFSET = 0x0464*8;
    private static final int PKMN_STORAGE_LENGTH = 555*362;
    private static final int ACTIVE_PKMN_OFFSET = 0x83D9*8+1;
    private static final int ACTIVE_PKMN_LENGTH = 4*546;
    private static final int ACTIVE_PKMN_EP_OFFSET = 0x84F4*8+2;
    private static final int ACTIVE_PKMN_EP_LENGTH = 4*546;
    private static final int ACTIVE_PKMN_FR_OFFSET = 0x860F*8+3;
    private static final int ACTIVE_PKMN_FR_LENGTH = 4*546;
    private static final int ADVENTURES_HAD_OFFSET = 0x8B70*8;
    private static final int ADVENTURES_HAD_LENGTH = 32;
    private static final int HELD_ITEM_STORAGE_OFFSET = 0x8BA2*8;
    private static final int HELD_ITEM_STORAGE_LENGTH = 50*33;
    private static final int HELD_ITEM_EP_STORAGE_OFFSET = 0x8C70*8+2;//HELD_ITEM_STORAGE_OFFSET + HELD_ITEM_STORAGE_LENGTH
    private static final int HELD_ITEM_EP_STORAGE_LENGTH = 50*33;//HELD_ITEM_STORAGE_LENGTH
    private static final int HELD_ITEM_FR_STORAGE_OFFSET = 0x8D3E*8+4;//HELD_ITEM_EP_STORAGE_OFFSET + HELD_ITEM_EP_STORAGE_LENGTH
    private static final int HELD_ITEM_FR_STORAGE_LENGTH = 50*33;//HELD_ITEM_STORAGE_LENGTH
    private static final int ITEM_STORAGE_OFFSET = 0x8E0C*8+6;
    private static final int ITEM_STORAGE_LENGTH = 1000*2*11;
    private static final int ITEM_SHOP1_OFFSET = 0x98CA*8+6;
    private static final int ITEM_SHOP1_LENGTH = 8*22;
    private static final int ITEM_SHOP2_OFFSET = 0x98E0*8+6;//ITEM_SHOP1_OFFSET + ITEM_SHOP1_LENGTH
    private static final int ITEM_SHOP2_LENGTH = 4*22;
    private static final int HELD_MONEY_OFFSET = 0x990C*8+6;
    private static final int HELD_MONEY_LENGTH = 17;
    private static final int HELD_MONEY_EP_OFFSET = 0x990E*8+7;//HELD_MONEY_OFFSET + HELD_MONEY_LENGTH
    private static final int HELD_MONEY_EP_LENGTH = 17;//HELD_MONEY_LENGTH
    private static final int HELD_MONEY_FR_OFFSET = 0x9912*8+6;
    private static final int HELD_MONEY_FR_LENGTH = 17;//HELD_MONEY_LENGTH
    private static final int STORED_MONEY_OFFSET = 0x9915*8+6;
    private static final int STORED_MONEY_LENGTH = 24;
    private static final int TEAMNAME_OFFSET = 0x994E*8;
    private static final int TEAMNAME_BYTELENGTH = 10;
    private static final int EGG_OFFSET = 0x9918*8+6;
    private static final int EGG_LENGTH = 32;
    private static final int ADVENTURE_LOG_OFFSET = 0x9958*8;
    private static final int ADVENTURE_LOG_LENGTH = 447; // TODO length
    private static final int CROAGUNK_SHOP_OFFSET = 0xB475*8;
    private static final int CROAGUNK_SHOP_LENGTH = 8*11;

    // private (public for debugging)
    public boolean[] split_data;
    public BooleanBuffer buf;

    public PkmnStorage pkmnStorage;
    public PkmnExStorage activePkmn;
    public PkmnExStorage activePkmnEP;
    public PkmnExStorage activePkmnFR; // FR - Friend Rescue
    public int adventuresHad; // signed
    public HeldItemStorage heldItemStorage;
    public HeldItemStorage heldItemEPStorage;
    public HeldItemStorage heldItemFRStorage; // FR - Friend Rescue
    public ItemStorage itemStorage;
    public ItemShop itemShop1;
    public ItemShop itemShop2;
    public int heldMoney;
    public int heldMoneyEP;
    public int heldMoneyFR; // FR - Friend Rescue
    public int storedMoney;
    public SkyStringTrash teamName;
    public SkyEgg egg;
    public AdventureLog adventureLog;
    public ItemShopNoParams croagunkShop;

    public SaveSlot(byte[] data) {
        if(calcChecksum(data) != BitConverterLE.readInt32(data, 0)) {
            System.err.println("WARNING: checksum mismatch");
        }
        split_data = BitConverterLE.splitBits(data);
        buf = new BooleanBufferArray(split_data);
        data = null; // do not use later

        if(! Arrays.equals(MAGIC_DATA, buf.seek(MAGIC_OFFSET).getBytes(MAGIC_BYTELENGTH))) {
            System.err.println("WARNING: invalid magic");
        }

        pkmnStorage = new PkmnStorage(buf.seek(PKMN_STORAGE_OFFSET).view(PKMN_STORAGE_LENGTH));
        activePkmn = new PkmnExStorage(buf.seek(ACTIVE_PKMN_OFFSET).view(ACTIVE_PKMN_LENGTH));
        activePkmnEP = new PkmnExStorage(buf.seek(ACTIVE_PKMN_EP_OFFSET).view(ACTIVE_PKMN_EP_LENGTH));
        activePkmnFR = new PkmnExStorage(buf.seek(ACTIVE_PKMN_FR_OFFSET).view(ACTIVE_PKMN_FR_LENGTH));
        adventuresHad = buf.seek(ADVENTURES_HAD_OFFSET).getInt(ADVENTURES_HAD_LENGTH);
        heldItemStorage = new HeldItemStorage(buf.seek(HELD_ITEM_STORAGE_OFFSET).view(HELD_ITEM_STORAGE_LENGTH));
        heldItemEPStorage = new HeldItemStorage(buf.seek(HELD_ITEM_EP_STORAGE_OFFSET).view(HELD_ITEM_EP_STORAGE_LENGTH));
        heldItemFRStorage = new HeldItemStorage(buf.seek(HELD_ITEM_FR_STORAGE_OFFSET).view(HELD_ITEM_FR_STORAGE_LENGTH));
        itemStorage = new ItemStorage(buf.seek(ITEM_STORAGE_OFFSET).view(ITEM_STORAGE_LENGTH));
        itemShop1 = new ItemShop(buf.seek(ITEM_SHOP1_OFFSET).view(ITEM_SHOP1_LENGTH));
        itemShop2 = new ItemShop(buf.seek(ITEM_SHOP2_OFFSET).view(ITEM_SHOP2_LENGTH));
        heldMoney = buf.seek(HELD_MONEY_OFFSET).getInt(HELD_MONEY_LENGTH);
        heldMoneyEP = buf.seek(HELD_MONEY_EP_OFFSET).getInt(HELD_MONEY_EP_LENGTH);
        heldMoneyFR = buf.seek(HELD_MONEY_FR_OFFSET).getInt(HELD_MONEY_FR_LENGTH);
        storedMoney = buf.seek(STORED_MONEY_OFFSET).getInt(STORED_MONEY_LENGTH);
        teamName = SkyCharConv.decode(buf.seek(TEAMNAME_OFFSET).getBytes(TEAMNAME_BYTELENGTH));
        egg = new SkyEgg(buf.seek(EGG_OFFSET).view(EGG_LENGTH));
        adventureLog = new AdventureLog(buf.seek(ADVENTURE_LOG_OFFSET).view(ADVENTURE_LOG_LENGTH));
        croagunkShop = new ItemShopNoParams(buf.seek(CROAGUNK_SHOP_OFFSET).view(CROAGUNK_SHOP_LENGTH));
    }

    public byte[] toByteA() {
        buf.seek(MAGIC_OFFSET).putBytes(MAGIC_DATA);

        pkmnStorage.store(buf.seek(PKMN_STORAGE_OFFSET).view(PKMN_STORAGE_LENGTH));
        activePkmn.store(buf.seek(ACTIVE_PKMN_OFFSET).view(ACTIVE_PKMN_LENGTH));
        activePkmnEP.store(buf.seek(ACTIVE_PKMN_EP_OFFSET).view(ACTIVE_PKMN_EP_LENGTH));
        activePkmnFR.store(buf.seek(ACTIVE_PKMN_FR_OFFSET).view(ACTIVE_PKMN_FR_LENGTH));
        buf.seek(ADVENTURES_HAD_OFFSET).putInt(adventuresHad, ADVENTURES_HAD_LENGTH);
        heldItemStorage.store(buf.seek(HELD_ITEM_STORAGE_OFFSET).view(HELD_ITEM_STORAGE_LENGTH));
        heldItemEPStorage.store(buf.seek(HELD_ITEM_EP_STORAGE_OFFSET).view(HELD_ITEM_EP_STORAGE_LENGTH));
        heldItemFRStorage.store(buf.seek(HELD_ITEM_FR_STORAGE_OFFSET).view(HELD_ITEM_FR_STORAGE_LENGTH));
        itemStorage.store(buf.seek(ITEM_STORAGE_OFFSET).view(ITEM_STORAGE_LENGTH));
        itemShop1.store(buf.seek(ITEM_SHOP1_OFFSET).view(ITEM_SHOP1_LENGTH));
        itemShop2.store(buf.seek(ITEM_SHOP2_OFFSET).view(ITEM_SHOP2_LENGTH));
        buf.seek(HELD_MONEY_OFFSET).putInt(heldMoney, HELD_MONEY_LENGTH);
        buf.seek(HELD_MONEY_EP_OFFSET).putInt(heldMoneyEP, HELD_MONEY_EP_LENGTH);
        buf.seek(HELD_MONEY_FR_OFFSET).putInt(heldMoneyFR, HELD_MONEY_FR_LENGTH);
        buf.seek(STORED_MONEY_OFFSET).putInt(storedMoney, STORED_MONEY_LENGTH);
        buf.seek(TEAMNAME_OFFSET).putBytes(SkyCharConv.encode(teamName));
        egg.store(buf.seek(EGG_OFFSET).view(EGG_LENGTH));
        adventureLog.store(buf.seek(ADVENTURE_LOG_OFFSET).view(ADVENTURE_LOG_LENGTH));
        croagunkShop.store(buf.seek(CROAGUNK_SHOP_OFFSET).view(CROAGUNK_SHOP_LENGTH));

        byte[] data = BitConverterLE.packBits(split_data);
        BitConverterLE.writeInt32(calcChecksum(data), data, 0);
        return data;
    }

    private static int calcChecksum(byte[] data) {
        int chksum = 0;
        for(int i=4; i<data.length; i+=4) {
            chksum += BitConverterLE.readInt32(data, i);
        }
        return chksum;
    }

}
